@@ -2,6 +2,7 @@ import logging
 import typing
 
 from corm.constants import CLUSTER_IPS, CLUSTER_PORT, PWN
+from corm.auth import AuthProvider
 from corm.encoders import DT_MAP
 from corm.models import CORMBase
 from corm.datatypes import CORMDetails, CassandraKeyspaceStrategy, TableOrdering
@@ -11,20 +12,33 @@ from cassandra.query import BatchStatement, SimpleStatement
 
 TABLES = {}
 SESSIONS = {}
-CLUSTER = Cluster(CLUSTER_IPS, port=CLUSTER_PORT)
+if AuthProvider:
+    CLUSTER = Cluster(CLUSTER_IPS, port=CLUSTER_PORT, auth_provider=AuthProvider)
+else:
+    CLUSTER = Cluster(CLUSTER_IPS, port=CLUSTER_PORT)
+
 SESSIONS['global'] = CLUSTER.connect()
 RESERVED_KEYSPACE_NAMES = ['global']
 
 logger = logging.getLogger(__name__)
 
-def obtain_session(keyspace_name: str) -> 'SESSIONS[keyspace_name]':
+def obtain_session(keyspace_name: str, auto_create_keyspace: bool = False) -> 'SESSIONS[keyspace_name]':
     if keyspace_name in RESERVED_KEYSPACE_NAMES:
         raise NotImplementedError(f'Unable to request Keyspace Name[{keyspace_name}]')
 
     if keyspace_name in SESSIONS.keys():
         return SESSIONS[keyspace_name]
 
-    SESSIONS[keyspace_name] = CLUSTER.connect(keyspace_name)
+    try:
+        SESSIONS[keyspace_name] = CLUSTER.connect(keyspace_name)
+    except Exception as err:
+        if auto_create_keyspace:
+            keyspace_create(keyspace_name, CassandraKeyspaceStrategy.Simple)
+            SESSIONS[keyspace_name] = CLUSTER.connect(keyspace_name)
+
+        else:
+            raise err
+
     return SESSIONS[keyspace_name]
 
 def keyspace_exists(keyspace_name: str) -> None:
@@ -54,7 +68,7 @@ def keyspace_destroy(keyspace_name: str) -> None:
     SESSIONS['global'].execute(CQL)
 
 def annihilate_keyspace_tables(keyspace_name: str) -> None:
-    FIND_TABLES_CQL = "SELECT table_name FROM system_schema.tables;"
+    FIND_TABLES_CQL = "SELECT table_name FROM system_schema.tables WHERE keyspace_name='{keyspace_name}';"
     for row in SESSIONS['global'].execute(FIND_TABLES_CQL):
         cql = f'DROP TABLE IF EXISTS {keyspace_name}.{row.table_name};'
         SESSIONS['global'].execute(cql)
@@ -104,7 +118,7 @@ def sync_schema() -> None:
             if keyspace_exists(keyspace_name) is False:
                 keyspace_create(keyspace_name, CassandraKeyspaceStrategy.Simple)
 
-        session = obtain_session(keyspace_name)
+        session = obtain_session(keyspace_name, True)
         for table in tables:
             COLUMN_CQL = f'''
             SELECT
@@ -115,10 +129,11 @@ def sync_schema() -> None:
                 AND keyspace_name = ?'''
             stmt = obtain_session(keyspace_name).prepare(COLUMN_CQL)
             existing_columns = {r.column_name: r.type for r in obtain_session(keyspace_name).execute(stmt, [table.table_name, keyspace_name])}
+
             # Add Whole Table
             if len(existing_columns.keys()) == 0:
-                logger.info(f'Creating Table[{table.table_name}]')
-                obtain_session(keyspace_name).execute(table.as_create_table_cql())
+                logger.info(f'Creating Table[{table.table_name}] in Keyspace[{table.keyspace}]')
+                session.execute(table.as_create_table_cql())
 
             # Add Columns
             elif len(table.field_names) > len(existing_columns.keys()) - 1:
@@ -206,3 +221,7 @@ class select:
             values.append(getattr(next_object, field_name, None))
 
         return self._table(*values)
+
+class where:
+    def __init__(self: PWN) -> None:
+        raise NotImplementedError
